﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TwoLiquidTanks;
using System.IO;
using System.Threading;
using System.Collections;

namespace TwoLiquidTanks
{

    public partial class Form1 : Form
    {
        private class MyRenderer : ToolStripProfessionalRenderer
        {
            public MyRenderer() : base(new MyColors()) { }
        }

        private class MyColors : ProfessionalColorTable
        {
            public override Color MenuItemSelected
            {
                get { return Color.Transparent; }
            }
            /*public override Color MenuItemSelectedGradientBegin
            {
                get { return Color.Orange; }
            }
            public override Color MenuItemSelectedGradientEnd
            {
                get { return Color.Yellow; }
            }*/

        }


        //---------------------------------------------------------
        //Parameters 1st cone
        //---------------------------------------------------------
        private int k;              // actual simulation step
        private int numo_step;  // number of simulation steps
        //private double _simulationTime;    // simulation time of control
        private double T0;      // sampling period [s]
        private double h1s;     // last steady state h1 [m]
        private double q1fs;    // last steady state q1f [m^3/s]
        private double[] h1;    // actual water level h1 [m]
        private double[] q1f;   // actual flowrate [m^3/s]
        private double[] u1;    // controller output u1
        private double[] u1f;   // filtered controller output u1
        private double[] w1;    // desired water level h1
        private double[] y1;    // deviation from last steady state h1
        private double[] y1f;   // filtered deviation from last steady state h1
        private double[] e1;    // difference w-y
        private string[][] simData;
        //---------------------------------------------------------
        //Parameters 2nd cone
        //---------------------------------------------------------
        private double h2s;     // last steady state h2 [m]
        private double q2fs;    //last steadz state q2f [m^3/s]
        private double[] h2;    // actual water level h2 [m]
        private double[] q2f;   // actual flowrate [m^3/s]
        private double[] u2;    // controller output u2
        private double[] u2f;   // filtered controller output u2
        private double[] w2;    // desired water level h2
        private double[] y2;    // deviation from last steady state h2
        private double[] y2f;   // filtered deviation from last steady state h2
        private double[] e2;    // difference w-y
        //---------------------------------------------------------
        //Other parameters
        //---------------------------------------------------------
        private double D;       // diameter m
        private double H;       // hight m
        private double k1;      // valve constant  [(m^5/2)/s]
        private double k2;      // valve constant  [(m^5/2)/s]
        private double[] q1;  //  flowrate through valve 1 [m^3/s]
        private double[] q2;  //  flowrate through valve 2 [m^3/s]  
        //---------------------------------------------------------
        // Identification parameters
        //---------------------------------------------------------
        private int param;
        private double[,] Theta_h1;
        private double[,] Theta_h2;
        private double[] Theta_backup;
        private double[,] c1;
        private double[,] c2;
        private double[,] d_h1;
        private double[,] d_h2;
        //private double ep1;
        //private double eps1;
        private double[] b01;
        private double[] a01;
        private double[] a02;
        //private double ep2;
        //private double eps2;
        private double[] b04;
        private double[] a03;
        private double[] a04;
        //---------------------------------------------------------
        //Control parameters
        //---------------------------------------------------------
        private double[] controller;
        private double maxLevelh1;
        private double maxLevelh2;
        private double alpha1;
        private double alpha2;
        private double gamma1;
        private double gamma2;
        private double[] z11;
        private double[] z14;
        private double[] UR;
        private double[] UQ;
        private double[] waterLevel1;
        private double[] waterLevel2;
        private double[] desiredTime1;
        private double[] desiredTime2;
        //---------------------------------------------------------
        // identification
        //---------------------------------------------------------
        private double _phi_coef;
        private int _selectedIdentificationType;    //LSM, LSM exponencial, LSM adaptive forgetting
        private int _selectedIdentification_spec;   //delta identif. x diferencial filters
        //adaptive directional forgetting variables
        private double phi_adaptive_h1;
        private double ro_adaptive_h1;
        private double lambda_adaptive_h1;
        private double ny_adaptive_h1;
        private double[] adaptiveDirectionalBackupParameters;
        //---------------------------------------------------------
        private double phi_adaptive_h2;
        private double ro_adaptive_h2;
        private double lambda_adaptive_h2;
        private double ny_adaptive_h2;
        // Diferencial filter constant CO
        private double filter_co;
        //---------------------------------------------------------
        //---------------------------------------------------------
        // pom variables
        //---------------------------------------------------------
        private double _actuator1;
        private double _acturator2;
        private double _simulationtime;

        //---------------------------------------------------------
        // export variables
        //---------------------------------------------------------
        private bool whichExport;
        //---------------------------------------------------------
        public Form1()
        {
            InitializeComponent();
        }
        private void InitTheta(double[,] M, double[] param, int rowSpace, int colSpace)
        {
            for (int i = 0; i < rowSpace; i++)
            {
                for (int j = 0; j < colSpace; j++)
                {
                    M[i, j] = param[j];
                }
            }
        }
        private void matrixEye(double[,] M, double Cx, int rowSpace, int colSpace)
        {
            for (int i = 0; i < rowSpace; i++)
            {
                for (int j = 0; j < colSpace; j++)
                {
                    if (i == j)
                    {
                        M[i, j] = Cx;
                    }
                    else
                    {
                        M[i, j] = 0;
                    }
                }
            }
        }
        private void matrixInitialize(double[,] M,double Cx, int rowSpace,int colSpace)
        {
            for (int i = 0;i < rowSpace;i++)
            {
                for(int j = 0; j < colSpace;j++)
                {
                    M[i,j] = Cx;
                }
            }
        }
        private double[,] matrixTranspose(double[,] M, int parRow, int parCol)
        {
            double[,] mNew = new double[parCol, parRow];
            for (int i = 0; i < parCol; i++)
            {
                for (int j = 0; j < parRow; j++)
                {
                    mNew[i,j] = M[j,i];
                }
            }
            return mNew;
        }
        private double[,] matrixMultiple(double[,] M1, double[,] M2, int parRowM1, int parColM1, int parRowM2, int parColM2)
        {
            double[,] M3 = new double[parRowM1, parColM2];
            matrixInitialize(M3, 0, parRowM1, parColM2);
            for (int i = 0; i < parRowM1; i++)
            {
                for (int j = 0; j < parColM2; j++)
                {
                    for (int k = 0; k < parRowM2; k++)
                    {
                        M3[i, j] = M3[i, j] + M1[i, k] * M2[k, j];
                    }
                }
            }
            return M3;
        }
        private double[,] matrixAdd(double[,] M1, double[,] M2, int parRowM1, int parColM1, int parRowM2, int parColM2,int flag)
        {
            double[,] M3= new double[parRowM1, parColM1];
                for (int i = 0; i < parRowM1; i++)
                {
                    for (int j = 0; j < parColM1; j++)
                    {
                        if (flag == 1)
                        {
                            M3[i, j] = M1[i, j] + M2[i, j];
                        }
                        else
                        {
                            M3[i, j] = M1[i, j] - M2[i, j];
                        }
                    }
                }    
            return M3;
        }
        private double[,] matrixConstant(double[,] M, double Cx, int parRow, int parCol)
        {
            for (int i = 0; i < parRow; i++)
            {
                for (int j = 0; j < parCol; j++)
                {
                    M[i, j] = M[i, j] * Cx;

                }
            }
            return M;
        }
        // 
        private double[,] updateTheta(double[,] pardH, double[,] cMatrix, double[] pary1, double[] pary2, double[] paru,double[] pary, double[,] parThetaH, int parStep,bool type,int parIdentType)
        {
            pardH[0, 0] = paru[parStep];    // Input u (flowrate 1_actual or flowrate 2_actual)
            pardH[0, 1] = -pary1[parStep];  // Output y1
            pardH[0, 2] = -pary2[parStep];  // Output y2
            double[,] ep = new double[1, 1];
            double[,] eps = new double[1, 1];
            double[,] delta_h = new double[1, 1];
            double[,] onesUpdate = new double[1, 1];
            double[,] result = new double[this.param,1];
            

            delta_h[0, 0] = (pary[parStep + 1] - pary[parStep]) / this.T0;


            ep = matrixAdd(delta_h, matrixMultiple(parThetaH, matrixTranspose(pardH, 1, this.param), 1, this.param, this.param, 1), 1, 1, 1, 1, 2);
            eps = matrixMultiple(pardH, matrixMultiple(cMatrix, matrixTranspose(pardH, 1, this.param), this.param, this.param, this.param, 1), 1, this.param, this.param, 1);
            switch (parIdentType)
            {
                case 0:
                    result = matrixAdd(matrixTranspose(parThetaH, 1, this.param), matrixConstant(matrixConstant(matrixMultiple(cMatrix, matrixTranspose(pardH, 1, this.param), this.param, this.param, this.param, 1), ep[0, 0], this.param, 1), (1 / (1 + eps[0, 0])), this.param, 1), this.param, 1, this.param, 1, 1);
                    break;
                case 1:
                    result = matrixAdd(matrixTranspose(parThetaH, 1, this.param), matrixConstant(matrixConstant(matrixMultiple(cMatrix, matrixTranspose(pardH, 1, this.param), this.param, this.param, this.param, 1), ep[0, 0], this.param, 1), (1 / ((_phi_coef)*(_phi_coef) + eps[0, 0])), this.param, 1), this.param, 1, this.param, 1, 1);
                    break;
                case 2:
                    result = matrixAdd(matrixTranspose(parThetaH, 1, this.param), matrixConstant(matrixConstant(matrixMultiple(cMatrix, matrixTranspose(pardH, 1, this.param), this.param, this.param, this.param, 1), ep[0, 0], this.param, 1), (1 / (1 + eps[0, 0])), this.param, 1), this.param, 1, this.param, 1, 1);
                    break;
                default:
                    break;
            }
            
            return result;
        }
        private double[,] updateCovariance(double[,] pardH, double[,] cMatrix, double[] pary1, double[] pary2, double[] paru, double[,] parThetaH, int parStep, int parIdentType)
        {
            pardH[0, 0] = paru[parStep];
            pardH[0, 1] = -pary1[parStep];
            pardH[0, 2] = -pary2[parStep];
            double[,] eps = new double[1, 1];
            double[,] onesUpdate = new double[1, 1];
            eps = matrixMultiple(pardH, matrixMultiple(cMatrix, matrixTranspose(pardH, 1, this.param), this.param, this.param, this.param, 1), 1, this.param, this.param, 1);
            switch (parIdentType)
            {
                case 0:
                    cMatrix = matrixAdd(cMatrix, matrixConstant(matrixMultiple(matrixMultiple(matrixMultiple(cMatrix, matrixTranspose(pardH, 1, this.param), this.param, this.param, this.param, 1), pardH, this.param, 1, 1, this.param), cMatrix, this.param, this.param, this.param, this.param), (1 / (1 + eps[0, 0])), this.param, this.param), this.param, this.param, this.param, this.param, 2);
                    break;
                case 1:
                    cMatrix = matrixConstant(matrixAdd(cMatrix, matrixConstant(matrixMultiple(matrixMultiple(matrixMultiple(cMatrix, matrixTranspose(pardH, 1, this.param), this.param, this.param, this.param, 1), pardH, this.param, 1, 1, this.param), cMatrix, this.param, this.param, this.param, this.param), (1 / ((_phi_coef) * (_phi_coef) + eps[0, 0])), this.param, this.param), this.param, this.param, this.param, this.param, 2), (1 / (_phi_coef*_phi_coef)), this.param, this.param);
                    break;
                default:
                    break;
            }
            
            return cMatrix;
        }
        private double[,] updateCovarianceAdaptive(double[,] pardH, double[,] cMatrix, double[] pary1, double[] pary2, double[] paru,double[] pary, double[,] parThetaH, int parStep, bool type,int parIdentType,ref double phi_adaptive_h,ref double lambda_adaptive_h, ref double ny_adaptive_h)
        {
            pardH[0, 0] = paru[parStep];
            pardH[0, 1] = -pary1[parStep];
            pardH[0, 2] = -pary2[parStep];
            double[,] delta_h = new double[1,1];
            double[,] ep = new double[1, 1];
            double[,] eps = new double[1, 1];
            double[,] epsilon = new double[1, 1];
            double eta_adaptive_h;

            delta_h[0, 0] = (pary[parStep + 1] - pary[parStep]) / this.T0;


            ep = matrixAdd(delta_h, matrixMultiple(parThetaH, matrixTranspose(pardH, 1, this.param), 1, this.param, this.param, 1), 1, 1, 1, 1, 2);

            eps = matrixMultiple(pardH, matrixMultiple(cMatrix, matrixTranspose(pardH, 1, this.param), this.param, this.param, this.param, 1), 1, this.param, this.param, 1);

            if (eps[0, 0] > 0)
            {
                epsilon[0, 0] = phi_adaptive_h - ((1 - phi_adaptive_h) / (eps[0, 0]));
                cMatrix = matrixAdd(cMatrix, matrixConstant(matrixMultiple(matrixMultiple(matrixMultiple(cMatrix, matrixTranspose(pardH, 1, this.param), this.param, this.param, this.param, 1), pardH, this.param, 1, 1, this.param), cMatrix, this.param, this.param, this.param, this.param), (1 / ((1/epsilon[0,0]) + eps[0, 0])), this.param, this.param), this.param, this.param, this.param, this.param, 2); ;
            }
            lambda_adaptive_h = phi_adaptive_h*(lambda_adaptive_h+(ep[0,0]*ep[0,0])/(1+eps[0,0]));
            ny_adaptive_h = phi_adaptive_h * (ny_adaptive_h + 1);
            eta_adaptive_h = (ep[0, 0] * ep[0, 0]) / (lambda_adaptive_h);           
            phi_adaptive_h = 1 / (1 + (1 + ro_adaptive_h1) * (Math.Log(1 + eps[0, 0] - eps[0, 0] / (1 + eps[0, 0]) + eps[0, 0] * (ny_adaptive_h2 + 1) * eta_adaptive_h / (1 + eps[0, 0] + eta_adaptive_h) / (1 + eps[0, 0]))));
            //textBox3.Text = (1 + eps[0, 0] - eps[0, 0] / (1 + eps[0, 0]) + eps[0, 0] * (ny_adaptive_h2 + 1) * eta_adaptive_h / (1 + eps[0, 0] + eta_adaptive_h) / (1 + eps[0, 0])).ToString();
            //textBox4.Text = eps[0,0].ToString();

            return cMatrix;
        }
        private bool exportCSV(int parStep,bool type)
        {
            if (type == false)
            {
                simData = new string[parStep + 2][];
                for (int x = 0; x <= parStep; x++)
                {
                    simData[x] = new string[16 + 1];
                }
                // copy  var
                for (int i = 0; i <= parStep; i++)
                {
                    simData[i][0] = (i * T0).ToString();

                    simData[i][1] = y1[i].ToString();
                    simData[i][2] = y2[i].ToString();

                    simData[i][3] = h1[i].ToString();
                    simData[i][4] = h2[i].ToString();

                    simData[i][5] = u1[i].ToString();
                    simData[i][6] = u2[i].ToString();

                    simData[i][7] = q1[i].ToString();
                    simData[i][8] = q2[i].ToString();

                    simData[i][9] = w1[i].ToString();
                    simData[i][10] = w2[i].ToString();

                    simData[i][11] = b01[i].ToString();
                    simData[i][12] = a01[i].ToString();
                    simData[i][13] = a02[i].ToString();

                    simData[i][14] = b04[i].ToString();
                    simData[i][15] = a03[i].ToString();
                    simData[i][16] = a04[i].ToString();

                }
            }
            else
            {
                simData = new string[parStep + 2][];
                for (int x = 0; x <= parStep; x++)
                {
                    simData[x] = new string[14 + 1];
                }
                for (int i = 0; i <= parStep; i++)
                {
                    simData[i][0] = (i * T0).ToString();

                    simData[i][1] = y1[i].ToString();
                    simData[i][2] = y2[i].ToString();

                    simData[i][3] = h1[i].ToString();
                    simData[i][4] = h2[i].ToString();

                    simData[i][5] = q1f[0].ToString();
                    simData[i][6] = q2f[0].ToString();

                    simData[i][7] = q1[i].ToString();
                    simData[i][8] = q2[i].ToString();

                    simData[i][9] = b01[i].ToString();
                    simData[i][10] = a01[i].ToString();
                    simData[i][11] = a02[i].ToString();

                    simData[i][12] = b04[i].ToString();
                    simData[i][13] = a03[i].ToString();
                    simData[i][14] = a04[i].ToString();

                }
            }
            return type;
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            this.BackColor = Color.FromArgb(1, 26, 38);
            this.Opacity = 0.7;
            this.tabPage1.BackColor = Color.FromArgb(1, 26, 38);
            this.tabPage2.BackColor = Color.FromArgb(1, 26, 38);
            this.groupBox1.BackColor = Color.FromArgb(1, 26, 38);
            this.groupBox3.BackColor = Color.FromArgb(1, 26, 38);
            this.trackBar3.BackColor = Color.FromArgb(1, 26, 38);
            this.trackBar4.BackColor = Color.FromArgb(1, 26, 38);
            this.radioButton4.BackColor = Color.FromArgb(1, 26, 38);
            this.radioButton5.BackColor = Color.FromArgb(1, 26, 38);
            menuStrip1.Renderer = new MyRenderer();
            //comboBox1.BackColor = Color.FromArgb(1, 26, 38);
            comboBox1.BackColor = Color.WhiteSmoke;
            //menuStrip1.BackColor = Color.FromArgb(3, 28, 39);
            //menuStrip1.i
            menuStrip1.Items["simulationParametersToolStripMenuItem"].Image = Image.FromFile("..\\..\\img\\cog.png");
            menuStrip1.Items["helpToolStripMenuItem1"].Image = Image.FromFile("..\\..\\img\\glass.png");
            menuStrip1.Items["helpToolStripMenuItem"].Image = Image.FromFile("..\\..\\img\\key.png");
            statusStrip1.Items["toolStripStatusLabel1"].Image = Image.FromFile("..\\..\\img\\clock.png");

            pictureBox1.BackgroundImage = Image.FromFile("..\\..\\img\\run_default.png");
            pictureBox1.BackgroundImageLayout = ImageLayout.Stretch;
            pictureBox2.BackgroundImage = Image.FromFile("..\\..\\img\\close_default.png");
            pictureBox2.BackgroundImageLayout = ImageLayout.Stretch;
            chart1.Series["Series1"].LegendText = "Deviation y1";
            chart1.Series["Series2"].LegendText = "Deviation y2";
            this.checkBox1.Checked = true;
            for (int i = 3; i <= 14; i++)
            {
                string pom = "Series" + i.ToString();
                chart1.Series[pom].Enabled = false;
            }

            ToolTip toolTip1 = new ToolTip();
            toolTip1.ShowAlways = true;
            toolTip1.SetToolTip(groupBox2, "LSM - least square method");
            toolStripProgressBar1.Minimum = 0;
            
            this.radioButton4.Checked = true;
            //numo_step =  800;
            _simulationtime = 100;
            T0 = 0.1;
            numo_step = (int)(_simulationtime / T0);
            param = 3;
            controller = new double[12];
            this.alpha1 = 0.4;
            this.alpha2 = 0.2;
            this.gamma1 = 0;
            this.gamma2 = 0;
            this.textBox3.Text = gamma1.ToString();
            this.textBox4.Text = gamma2.ToString();
            this.textBox1.Text = alpha1.ToString();
            this.textBox2.Text = alpha2.ToString();
            toolStripProgressBar1.Maximum = (int)(_simulationtime/T0);            
            h1s = 1.8;
            h2s = 1.4;
            q1fs = 0.20; // 60;
            q2fs = 0.15; // 60;
            D = 1.5;
            H = 2.5;
            k1 = 0.316;// 60;
            k2 = 0.296;// 60;
            //simData = new double[numo_step, 15];
            

            h1 = new double[numo_step+2];
            h1[0] = h1s;
            u1 = new double[numo_step+2];
            u1[0] = 0;
            u1f = new double[numo_step + 2];
            u1f[0] = 0;
            q1f = new double[numo_step+2];
            q1f[0] = q1fs+u1[0];
            //w1 = new double[numo_step+2];
            //w1[0] = 0.2;
            y1 = new double[numo_step+2];
            y1[0] = 0;
            y1f = new double[numo_step + 2];
            y1f[0] = 0;
            e1 = new double[numo_step+2];
            //e1[0] = w1[0] - y1[0];
            h2 = new double[numo_step+2];
            h2[0] = h2s;
            u2 = new double[numo_step+2];
            u2[0] = 0;
            u2f = new double[numo_step + 2];
            u2f[0] = 0;
            q2f = new double[numo_step+2];
            q2f[0] = q2fs+u2[0];
            //w2 = new double[numo_step+2];
            //w2[0] = 0.1;
            y2 = new double[numo_step+2];
            y2[0] = 0;
            e2 = new double[numo_step+2];
            //e2[0] = w2[0] - y2[0];
            q1 = new double[numo_step+2];
            q2 = new double[numo_step+2];
            q1[0] = k1 * Math.Sign(h1[0] - h2[0]) * Math.Sqrt(Math.Abs(h1[0] - h2[0]));
            q2[0] = k2 * Math.Sqrt(h2[0]);
            //--------------------------------------------------------------------------
            //Controller design
            //--------------------------------------------------------------------------
            z11 = new double[numo_step + 2];
            z14 = new double[numo_step + 2];
            z11[0] = 0;
            z14[0] = 0;
            UR = new double[2];
            UQ = new double[2];
            //--------------------------------------------------------------------------
            //Identification parameters initialization
            _phi_coef = 0.9;
            initialAdaptiveForgetting();

            
            //--------------------------------------------------------------------------

            chart1.ChartAreas["ChartArea1"].AxisX.Title = "t [min]";
            chart1.Series["Series1"].Points.AddXY(0,1.8);
            chart1.Series["Series2"].Points.AddXY(0, 1.4);
            chart1.ChartAreas["ChartArea1"].AxisX.Minimum = 0;
            chart1.ChartAreas["ChartArea1"].AxisX.Maximum = _simulationtime;
            chart1.Series["Series1"].BorderWidth = 3;
            chart1.Series["Series2"].BorderWidth = 3;
            chart1.Series["Series3"].BorderWidth = 3;
            chart1.Series["Series4"].BorderWidth = 3;
            chart1.Series["Series5"].BorderWidth = 3;
            chart1.Series["Series6"].BorderWidth = 3;
            chart1.Series["Series7"].BorderWidth = 3;
            chart1.Series["Series7"].BorderDashStyle = System.Windows.Forms.DataVisualization.Charting.ChartDashStyle.DashDot;
            chart1.Series["Series8"].BorderWidth = 3;
            chart1.Series["Series8"].BorderDashStyle = System.Windows.Forms.DataVisualization.Charting.ChartDashStyle.DashDot;
            //chart1.Series["Series1"]
            initialThetaBackup();
            Theta_h1 = new double[1,param];
            InitTheta(Theta_h1, new double[] { Theta_backup[0], Theta_backup[1], Theta_backup[2] }, 1, param);
            Theta_h2 = new double[1,param];
            InitTheta(Theta_h2, new double[] { Theta_backup[3], Theta_backup[4], Theta_backup[5] }, 1, param);
            c1 = new double[param, param];
            c2 = new double[param, param];
            matrixEye(c1, 1000, this.param, this.param);
            matrixEye(c2, 1000, this.param, this.param);
            d_h1 = new double[1, param];
            d_h2 = new double[1, param];

            //initialize identification par
            // tank 1
            b01 = new double[numo_step+2];
            b01[0] = Theta_h1[0,0];
            a01 = new double[numo_step+2];
            a01[0] = Theta_h1[0,1];
            a02 = new double[numo_step+2];
            a02[0] = Theta_h1[0,2];
            // tank 2
            b04 = new double[numo_step+2];
            b04[0] = Theta_h2[0,1];
            a03 = new double[numo_step+2];
            a03[0] = Theta_h2[0,1];
            a04 = new double[numo_step+2];
            a04[0] = Theta_h2[0,2];

            filter_co = 0.5;

        }
        private void initialIdentification()
        {
            _selectedIdentificationType = selectedRadioButton();
        }
        private void initialThetaBackup()
        {
            Theta_backup = new double[6];
            Theta_backup[0] = 1.0083;   //b01
            Theta_backup[1] = 0.2373;   //a01
            Theta_backup[2] = -0.2356;  //a02
            Theta_backup[3] =  1.6864;  //b04
            Theta_backup[4] = -0.3856;  //a03
            Theta_backup[5] =  0.5869;  //a04

        }
        private void initialAdaptiveForgetting()
        {

            adaptiveDirectionalBackupParameters = new double[] { 1, 0.001, 0.99, 0.000001 };
            phi_adaptive_h1 = adaptiveDirectionalBackupParameters[0];
            ro_adaptive_h1 = adaptiveDirectionalBackupParameters[2];
            lambda_adaptive_h1 = adaptiveDirectionalBackupParameters[1];
            ny_adaptive_h1 = adaptiveDirectionalBackupParameters[3];
           
            //---------------------------------------------------------
            phi_adaptive_h2 = phi_adaptive_h1;
            ro_adaptive_h2 = ro_adaptive_h1;
            lambda_adaptive_h2 = lambda_adaptive_h1;
            ny_adaptive_h2 = ny_adaptive_h1;
        }
        private void initialAdaptiveControlling()
        {
            phi_adaptive_h1 = adaptiveDirectionalBackupParameters[0];
            ro_adaptive_h1 = adaptiveDirectionalBackupParameters[2];
            lambda_adaptive_h1 = adaptiveDirectionalBackupParameters[1];
            ny_adaptive_h1 = adaptiveDirectionalBackupParameters[3];

            phi_adaptive_h2 = phi_adaptive_h1;
            ro_adaptive_h2 = ro_adaptive_h1;
            lambda_adaptive_h2 = lambda_adaptive_h1;
            ny_adaptive_h2 = ny_adaptive_h1;

        }
        private void initialControlling()
        {
            bool contr = true;
            double member;
            foreach (Control c in groupBox3.Controls)
            {
                if (c.GetType() == typeof(TextBox))
                {
                    TextBox tb = c as TextBox;
                    if (double.TryParse(tb.Text.ToString(),out member) == false)
                    {
                        contr = false;
                    }
                }
            }
            if (contr == true)
            {
                gamma1 = double.Parse(textBox3.Text);
                gamma2 = double.Parse(textBox4.Text);
                alpha1 = double.Parse(textBox1.Text);
                alpha2 = double.Parse(textBox2.Text);
            }

            maxLevelh1 = 0.7;
            maxLevelh2 = 1.1;
            string wL1 = this.textBox5.Text;
            wL1 = wL1.Replace(" ", "");
            string wL2 = this.textBox6.Text;
            wL2 = wL2.Replace(" ", "");
            string dT1 = this.textBox7.Text;
            dT1 = dT1.Replace(" ", "");
            string dT2 = this.textBox8.Text;
            dT2 = dT2.Replace(" ", "");
            //waterLevel1 = Array.ConvertAll(wL1.Split('-'), Double.Parse);
            //waterLevel2 = Array.ConvertAll(wL2.Split('-'), Double.Parse);
            //desiredTime1 = Array.ConvertAll(dT1.Split('-'), Double.Parse);
            //desiredTime2 = Array.ConvertAll(dT2.Split('-'), Double.Parse);
            
        }
        private void desiredLevels()
        {
            int index1 = 0;
            int index2 = 0;
            for (int i = 0; i <= numo_step; i++) 
            {
                //water Level1
                if (i * T0 == desiredTime1[index1])
                {
                    w1[i] = waterLevel1[index1];
                    if (!(index1 == desiredTime1.Length))
                    {
                        index1++;
                    }
                }
                w1[i + 1] = w1[i];
                //water Level2
                if (i * T0 == desiredTime2[index2])
                {
                    w2[i] = waterLevel2[index2];
                    if (!(index2 == desiredTime2.Length))
                    {
                        index2++;
                    }
                }
                w2[i + 1] = w2[i];
            }
        }
        /*private void button1_Click(object sender, EventArgs e)
        {
            this.k = 0;
            if (radioButton5.Checked == true)
            {
                initialControlling();
                //desiredLevels();
            }
            InitTheta(Theta_h1, new double[] { 1.0083, 0.2373, -0.2356 }, 1, param);
            matrixEye(c1, 1000, this.param, this.param);
            InitTheta(Theta_h2, new double[] { 1.6864, -0.3856, 0.5869 }, 1, param);
            matrixEye(c2, 1000, this.param, this.param);
            chart1.Series["Series1"].Points.Clear();
            chart1.Series["Series2"].Points.Clear();
            chart1.Series["Series3"].Points.Clear();
            chart1.Series["Series4"].Points.Clear();
            chart1.Series["Series5"].Points.Clear();
            chart1.Series["Series6"].Points.Clear();
            chart1.Series["Series7"].Points.Clear();
            chart1.Series["Series8"].Points.Clear();
            timer1.Enabled = true;
        }
        */
        //private void trackBar1_Scroll(object sender, EventArgs e)
        //{
         //   string input = Microsoft.VisualBasic.Interaction.InputBox("Prompt", "Title", "Default", 0, 0);
//
 //       }


        private void trackBar3_Scroll(object sender, EventArgs e)
        {
            _actuator1 = double.Parse(this.trackBar3.Value.ToString()) / 100;
            //q1f[0] = q1fs + u1[0];
            double up = Math.Round(double.Parse(this.trackBar3.Value.ToString()) / 100, 5);
            this.label7.Text = up.ToString();
               
        }

        private void radioButton5_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton5.Checked == true)
            {
                this.textBox5.Text = "0-40-40-80";
                this.textBox6.Text = "0,2-0,2-0,1-0,1";
                this.textBox7.Text = "0-40-40-80";
                this.textBox8.Text = "0,1-0,1-0-0";
                this.textBox1.Text = "0,4";
                this.textBox2.Text = "0,2";
                this.textBox3.Text = "0";
                this.textBox4.Text = "0";
            }
            else
            {
                this.textBox1.Text = "";
                this.textBox2.Text = "";
                this.textBox3.Text = "";
                this.textBox4.Text = "";
                this.textBox5.Text = "";
                this.textBox6.Text = "";
                this.textBox7.Text = "";
                this.textBox8.Text = "";            
            }
        }

        private void trackBar4_Scroll(object sender, EventArgs e)
        {
            _acturator2 = double.Parse(this.trackBar4.Value.ToString()) / 100;
            //q2f[0] = q2fs + u2[0];
            double up = Math.Round(double.Parse(this.trackBar4.Value.ToString()) / 100, 5);
            this.label9.Text = up.ToString();
        }

        /*private void button2_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        */
        private void fixedStepSizeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string value = this.T0.ToString();
            double member;
            if (InputBox.Show("Fixed step size",
                "&Insert  fixed size step: [min]", ref value) == DialogResult.OK)
            {
                if (double.TryParse(value.ToString(),out member) == true)
                {
                    this.T0 = double.Parse((Math.Round(double.Parse(value), 2).ToString()));
                }
            }
        }

        private void simulationTimeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string value = this._simulationtime.ToString();
            double member;
            if (InputBox.Show("Simulation time",
                "&Insert simulation time: [min]", ref value) == DialogResult.OK)
            {
                if(double.TryParse(value.ToString(),out member)==true)
                {
                    this._simulationtime = double.Parse((Math.Round(double.Parse(value), 2).ToString()));
                    chart1.ChartAreas["ChartArea1"].AxisX.Minimum = 0;
                    chart1.ChartAreas["ChartArea1"].AxisX.Maximum = _simulationtime;
                    toolStripProgressBar1.Maximum = (int)(_simulationtime / T0);
                    chart1.ChartAreas["ChartArea1"].RecalculateAxesScale();

                }


                
                //chart1.ChartAreas["ChartArea1"].AxisX.Minimum = 0;
                
            }
        }

        private void dataExportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string fileName ="export.csv";

            if (InputBox.Show("Export data",
                "&Insert name of file:", ref fileName) == DialogResult.OK)
            {
                string filePath = "..\\..\\export\\"+fileName;  
	            string delimiter = "    ";  
	 
	            /*string[][] output = new string[][]{  
	                new string[]{"Col 1 Row 1", "Col 2 Row 1", "Col 3 Row 1"},  
	                new string[]{"Col1 Row 2", "Col2 Row 2", "Col3 Row 2"}  
	            };*/
                string[][] headTitle;
                if (whichExport == false)
                {
                    headTitle = new string[][]{  
	                new string[]{"t","y1", "y2", "h1","h2","u1","u2","q1","q2","w1","w2","b01","a01","a02","b04","a03","a04"}};
                }
                else
                {
                    headTitle = new string[][]{  
	                new string[]{"t","y1", "y2", "h1","h2","q1f","q2f","q1","q2","b01","a01","a02","b04","a03","a04"}};
                }

                try
                {
                    int length = simData.GetLength(0);
                    StringBuilder sb = new StringBuilder();
                    sb.AppendLine(string.Join(delimiter, headTitle[0]));
                    for (int index = 0; index < (length - 1); index++)
                        sb.AppendLine(string.Join(delimiter, simData[index]));
                    File.WriteAllText(filePath, sb.ToString());
                }
                catch (Exception excep)
                {
                    WarnDialog1.Show("Warn message",
                "&"+excep.ToString().Remove(excep.ToString().IndexOf(":")));
                    
                }
            }
        }
        private int chckList()
        {
            int numOfSig = 5;
            bool[] arSignal = new bool[numOfSig];
            for (int i = 0; i < arSignal.Length; i++)
            {
                switch (i)
                {
                    case 0:
                        arSignal[i] = this.checkBox1.Checked;
                        break;
                    case 1:
                        arSignal[i] = this.checkBox2.Checked;
                        break;
                    case 2:
                        arSignal[i] = this.checkBox3.Checked;
                        break;
                    case 3:
                        arSignal[i] = this.checkBox4.Checked;
                        break;
                    case 4:
                        arSignal[i] = this.checkBox5.Checked;
                        break;
                    default:
                        break;
                }
            }
            int states = 0;
            for (int ind = 0; ind < numOfSig; ind++)
            {
                if (arSignal[ind] == false)
                    states += 1;
            }
                return states;

        }
        private void updateChart(int parSignals)
        {
            switch (parSignals)
            {
                case 1:
                    chart1.ChartAreas["ChartArea1"].Position.Y = 25;
                    chart1.ChartAreas["ChartArea1"].Position.Height = 75;
                    break;
                case 2:
                    chart1.ChartAreas["ChartArea1"].Position.Y = 20;
                    chart1.ChartAreas["ChartArea1"].Position.Height = 80;
                    break;
                case 3:
                    chart1.ChartAreas["ChartArea1"].Position.Y = 15;
                    chart1.ChartAreas["ChartArea1"].Position.Height = 85;
                    break;
                case 4:
                    chart1.ChartAreas["ChartArea1"].Position.Y = 10;
                    chart1.ChartAreas["ChartArea1"].Position.Height = 90;
                    break;
                case 5:
                    chart1.ChartAreas["ChartArea1"].Position.Y = 10;
                    chart1.ChartAreas["ChartArea1"].Position.Height = 90;
                    break;
                default:
                    MessageBox.Show("Undefined");
                    break;
            }
        }
        private bool checkControl(int numCheck)
        {
            int numOfSignals = 5;
            bool flag;
            if (numOfSignals == numCheck)
                flag = false;
            else
                flag = true;
            return flag;
        }
        private void updateAxisY()
        {
            string[] vocLabel = new string[5];
            vocLabel[0] = "y(t)";
            vocLabel[1] = "h(t)";
            vocLabel[2] = "u(t)";
            vocLabel[3] = "w(t)";
            vocLabel[4] = "e.par.(t)";
            string label = "";
            int numOfSig = 5;
            bool[] arSignal = new bool[numOfSig];
            for (int i = 0; i < arSignal.Length; i++)
            {
                switch (i)
                {
                    case 0:
                        arSignal[i] = this.checkBox1.Checked;
                        break;
                    case 1:
                        arSignal[i] = this.checkBox2.Checked;
                        break;
                    case 2:
                        arSignal[i] = this.checkBox3.Checked;
                        break;
                    case 3:
                        arSignal[i] = this.checkBox4.Checked;
                        break;
                    case 4:
                        arSignal[i] = this.checkBox5.Checked;
                        break;
                    default:
                        break;
                }
            }
            for(int inx = 0; inx < numOfSig;inx++)
            {
                if (arSignal[inx] == true)
                {
                    if (label == "")
                        label = label + vocLabel[inx];
                    else
                    {
                        label = label + ", " + vocLabel[inx]; 
                    }
                }
            }
            chart1.ChartAreas["ChartArea1"].AxisY.Title = label;
        }
        // return selected radioButtons in groupBox1

        private int selectedRadioButton()
        {
            int i = 0;
            foreach (Control c in groupBox2.Controls)
            {
                if (c.GetType() == typeof(RadioButton))
                {
                    RadioButton rb = c as RadioButton;
                    if (rb.Checked == true)
                    {
                        i = int.Parse(rb.Name.ToString().Replace("radioButton", ""));
                    }
                }
            }           
            return (i-1);
        }
        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (checkControl(chckList()) == false)
            {
                //MessageBox.Show("Select at least one type of signals");
                WarnDialog1.Show("Warn message",
                "&Select at least one type of signal");


                this.checkBox1.Checked = true;
            }
            else
            {
                updateChart(chckList());
                updateAxisY();
                if (this.checkBox1.Checked == false)
                {
                    chart1.Series["Series1"].Enabled = false;
                    chart1.Series["Series2"].Enabled = false;
                    chart1.ChartAreas["ChartArea1"].RecalculateAxesScale();
                }
                else
                {
                    chart1.Series["Series1"].Enabled = true;
                    chart1.Series["Series2"].Enabled = true;
                    chart1.ChartAreas["ChartArea1"].RecalculateAxesScale();
                }
            }
        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            chart1.Series["Series3"].LegendText = "Water level h1";
            chart1.Series["Series4"].LegendText = "Water level h2";
            if (checkControl(chckList()) == false)
            {
                MessageBox.Show("Select at least one type of signals");
                this.checkBox2.Checked = true;
            }
            else
            {
                updateChart(chckList());
                updateAxisY();
                if (this.checkBox2.Checked == false)
                {
                    chart1.Series["Series3"].Enabled = false;
                    chart1.Series["Series4"].Enabled = false;
                    chart1.ChartAreas["ChartArea1"].RecalculateAxesScale();
                }
                else
                {
                    chart1.Series["Series3"].Enabled = true;
                    chart1.Series["Series4"].Enabled = true;
                    chart1.ChartAreas["ChartArea1"].RecalculateAxesScale();
                }
            }
        }

        private void checkBox3_CheckedChanged(object sender, EventArgs e)
        {
            chart1.Series["Series5"].LegendText = "Controller output u1";
            chart1.Series["Series6"].LegendText = "Controller output u2";
            if (checkControl(chckList()) == false)
            {
                MessageBox.Show("Select at least one type of signals");
                this.checkBox3.Checked = true;
            }
            else
            {
                updateChart(chckList());
                updateAxisY();
                if (this.checkBox3.Checked == false)
                {
                    chart1.Series["Series5"].Enabled = false;
                    chart1.Series["Series6"].Enabled = false;
                    chart1.ChartAreas["ChartArea1"].RecalculateAxesScale();
                }
                else
                {
                    chart1.Series["Series5"].Enabled = true;
                    chart1.Series["Series6"].Enabled = true;
                    chart1.ChartAreas["ChartArea1"].RecalculateAxesScale();
                }
            }
        }


        private void checkBox4_CheckedChanged(object sender, EventArgs e)
        {
            chart1.Series["Series7"].LegendText = "Desired water level w1";
            chart1.Series["Series8"].LegendText = "Desired water level w2";
            if (checkControl(chckList()) == false)
            {
                MessageBox.Show("Select at least one type of signals");
                this.checkBox4.Checked = true;
            }
            else
            {
                updateChart(chckList());
                updateAxisY();
                if (this.checkBox4.Checked == false)
                {
                    chart1.Series["Series7"].Enabled = false;
                    chart1.Series["Series8"].Enabled = false;
                    chart1.ChartAreas["ChartArea1"].RecalculateAxesScale();
                }
                else
                {
                    chart1.Series["Series7"].Enabled = true;
                    chart1.Series["Series8"].Enabled = true;
                    chart1.ChartAreas["ChartArea1"].RecalculateAxesScale();
                }
                updateChart(chckList());
            }
        }

        private void checkBox5_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void timer1_Tick(object sender, EventArgs e)
        {

            switch(radioButton4.Checked)
            {
                case true:  //System identification
                    radioButton5.Enabled = false;
                    if (k <= (int)(_simulationtime/T0))
                    {
                        
                        try
                        {

                            h1[k + 1] = h1[k] + T0 * ((q1f[k] - q1[k]) / ((Math.PI * (Math.Pow(h1[k], 2)) * (Math.Pow(D, 2))) / (4 * (Math.Pow(H, 2)))));
                            y1[k + 1] = h1[k + 1] - h1s;
                            u1[k + 1] = u1[k];
                            q1f[k + 1] = this.q1fs + u1[k];
                            h2[k + 1] = h2[k] + T0 * (q2f[k] - q2[k] + q1[k]) / (Math.PI * (Math.Pow(h2[k], 2)) * (Math.Pow(D, 2)) / (4 * (Math.Pow(H, 2))));
                            //this.textBox4.Text = h2[2].ToString();

                            y2[k + 1] = h2[k + 1] - h2s;
                            u2[k + 1] = u2[k];
                            q2f[k + 1] = this.q2fs + u2[k];                           

                        }
                        catch (Exception exc)
                        {
                            timer1.Enabled = false;
                            WarnDialog1.Show("Warn message",
                                "&" + exc.ToString().Remove(exc.ToString().IndexOf(":")));
                            
                        }
                        q1[k + 1] = k1 * Math.Sign(h1[k + 1] - h2[k + 1]) * Math.Sqrt(Math.Abs(h1[k + 1] - h2[k + 1]));
                        q2[k + 1] = k2 * Math.Sqrt(h2[k + 1]);

                        chart1.Series["Series1"].Points.AddXY((k * T0), y1[k]);
                        chart1.Series["Series2"].Points.AddXY((k * T0), y2[k]);
                        chart1.Series["Series3"].Points.AddXY((k * T0), h1[k]);
                        chart1.Series["Series4"].Points.AddXY((k * T0), h2[k]);

                        if (_selectedIdentification_spec == 0)
                        {
                            switch (_selectedIdentificationType)
                            {

                                case 0:
                                    Theta_h1 = matrixTranspose(updateTheta(d_h1, c1, y1, y2, u1, y1, Theta_h1, k, true, _selectedIdentificationType), this.param, 1);
                                    c1 = updateCovariance(d_h1, c1, y1, y2, u1, Theta_h1, k, _selectedIdentificationType);
                                    Theta_h2 = matrixTranspose(updateTheta(d_h2, c2, y1, y2, u2, y2, Theta_h2, k, true, _selectedIdentificationType), this.param, 1);
                                    c2 = updateCovariance(d_h2, c2, y1, y2, u2, Theta_h2, k, _selectedIdentificationType);

                                    break;
                                case 1:
                                    Theta_h1 = matrixTranspose(updateTheta(d_h1, c1, y1, y2, u1, y1, Theta_h1, k, true, _selectedIdentificationType), this.param, 1);
                                    c1 = updateCovariance(d_h1, c1, y1, y2, u1, Theta_h1, k, _selectedIdentificationType);
                                    Theta_h2 = matrixTranspose(updateTheta(d_h2, c2, y1, y2, u2, y2, Theta_h2, k, true, _selectedIdentificationType), this.param, 1);
                                    c2 = updateCovariance(d_h2, c2, y1, y2, u2, Theta_h2, k, _selectedIdentificationType);
                                    
                                    break;
                                case 2:
                                    double[,] Theta_cop1 = Theta_h1;
                                    double[,] Theta_cop2 = Theta_h2;
                                    Theta_h1 = matrixTranspose(updateTheta(d_h1, c1, y1, y2, u1, y1, Theta_h1, k, true, _selectedIdentificationType), this.param, 1);
                                    c1 = updateCovarianceAdaptive(d_h1, c1, y1, y2, u1, y1, Theta_cop1, k, true, _selectedIdentificationType, ref phi_adaptive_h1, ref lambda_adaptive_h1, ref ny_adaptive_h1);
                                    Theta_h2 = matrixTranspose(updateTheta(d_h2, c2, y1, y2, u2, y2, Theta_h2, k, true, _selectedIdentificationType), this.param, 1);
                                    c2 = updateCovarianceAdaptive(d_h2, c2, y1, y2, u2, y2, Theta_cop2, k, true, _selectedIdentificationType, ref phi_adaptive_h2, ref lambda_adaptive_h2, ref ny_adaptive_h2);
                                    break;
                                default:
                                    break;
                            }
                        }
                        else
                        {

                            try
                            {
                                u1f[k + 1] = u1f[k] + T0 * (u1[k] - filter_co * u1f[k]);
                                u2f[k + 1] = u2f[k] + T0 * (u2[k] - filter_co * u2f[k]);
                                y1f[k + 1] = y1f[k] + T0 * (y1[k] - filter_co * y1f[k]);
                                y2f[k + 1] = y2f[k] + T0 * (y2[k] - filter_co * y2f[k]);
                            }
                            catch(Exception ee)
                            {
                                timer1.Enabled = false;
                                // Chybova hlaska
                            }
                            switch (_selectedIdentificationType)
                            {
                                case 0:
                                    Theta_h1 = matrixTranspose(updateTheta(d_h1, c1, y1f, y2f, u1f, y1f, Theta_h1, k, true, _selectedIdentificationType), this.param, 1);
                                    c1 = updateCovariance(d_h1, c1, y1f, y2f, u1f, Theta_h1, k, _selectedIdentificationType);
                                    Theta_h2 = matrixTranspose(updateTheta(d_h2, c2, y1f, y2f, u2f, y2f, Theta_h2, k, true, _selectedIdentificationType), this.param, 1);
                                    c2 = updateCovariance(d_h2, c2, y1f, y2f, u2f, Theta_h2, k, _selectedIdentificationType);
                                    break;
                                case 1:

                                    Theta_h1 = matrixTranspose(updateTheta(d_h1, c1, y1f, y2f, u1f, y1f, Theta_h1, k, true, _selectedIdentificationType), this.param, 1);
                                    c1 = updateCovariance(d_h1, c1, y1f, y2f, u1f, Theta_h1, k, _selectedIdentificationType);
                                    Theta_h2 = matrixTranspose(updateTheta(d_h2, c2, y1f, y2f, u2f, y2f, Theta_h2, k, true, _selectedIdentificationType), this.param, 1);
                                    c2 = updateCovariance(d_h2, c2, y1f, y2f, u2f, Theta_h2, k, _selectedIdentificationType);
                                    break;
                                case 2:
                                    double[,] Theta_cop1 = Theta_h1;
                                    double[,] Theta_cop2 = Theta_h2;

                                    Theta_h1 = matrixTranspose(updateTheta(d_h1, c1, y1f, y2f, u1f, y1f, Theta_h1, k, true, _selectedIdentificationType), this.param, 1);
                                    c1 = updateCovarianceAdaptive(d_h1, c1, y1f, y2f, u1f, y1f, Theta_cop1, k, true, _selectedIdentificationType, ref phi_adaptive_h1, ref lambda_adaptive_h1, ref ny_adaptive_h1);
                                    Theta_h2 = matrixTranspose(updateTheta(d_h2, c2, y1f, y2f, u2f, y2f, Theta_h2, k, true, _selectedIdentificationType), this.param, 1);
                                    c2 = updateCovarianceAdaptive(d_h2, c2, y1f, y2f, u2f, y2f, Theta_cop2, k, true, _selectedIdentificationType, ref phi_adaptive_h2, ref lambda_adaptive_h2, ref ny_adaptive_h2);

                                    break;
                                default:
                                    break;
                            }

                        }
                        
                        // Identifikovane parametry jednotlivych Theta
                        b01[k + 1] = Theta_h1[0, 0];
                        a01[k + 1] = Theta_h1[0, 1];
                        a02[k + 1] = Theta_h1[0, 2];

                        b04[k + 1] = Theta_h2[0, 0];
                        a03[k + 1] = Theta_h2[0, 1];
                        a04[k + 1] = Theta_h2[0, 2];


                        this.toolStripProgressBar1.PerformStep();
                        this.k = this.k + 1;
                    }
                    else
                    {
                        whichExport= exportCSV((int)(_simulationtime / T0), true);
                        toolStripProgressBar1.Value = toolStripProgressBar1.Minimum;
                        radioButton4.Enabled = true;
                        radioButton5.Enabled = true;
                        timer1.Enabled = false;
                    }
                    break;
                case false: //System controlling
                    radioButton4.Enabled = false;
                    if (k <=(int)(_simulationtime/T0))
                    {
                        
                        try
                        {
                            h1[k + 1] = h1[k] + T0 * ((q1f[k] - q1[k]) / ((Math.PI * (Math.Pow(h1[k], 2)) * (Math.Pow(D, 2))) / (4 * (Math.Pow(H, 2)))));
                            y1[k + 1] = h1[k + 1] - h1s;

                            h2[k + 1] = h2[k] + T0 * (q2f[k] - q2[k] + q1[k]) / (Math.PI * (Math.Pow(h2[k], 2)) * (Math.Pow(D, 2)) / (4 * (Math.Pow(H, 2))));
                            y2[k + 1] = h2[k + 1] - h2s;


                            q1[k + 1] = k1 * Math.Sign(h1[k + 1] - h2[k + 1]) * Math.Sqrt(Math.Abs(h1[k + 1] - h2[k + 1]));
                            q2[k + 1] = k2 * Math.Sqrt(h2[k + 1]);

                            
                        }
                        catch(Exception exc)
                        {
                            timer1.Enabled = false;
                            WarnDialog1.Show("Warn message",
                               "&" + exc.ToString().Remove(exc.ToString().IndexOf(":")));
                        }


                        chart1.Series["Series1"].Points.AddXY((k * T0), y1[k]);
                        chart1.Series["Series2"].Points.AddXY((k * T0), y2[k]);
                        chart1.Series["Series3"].Points.AddXY((k * T0), h1[k]);
                        chart1.Series["Series4"].Points.AddXY((k * T0), h2[k]);
                        chart1.Series["Series5"].Points.AddXY((k * T0), u1[k]);
                        chart1.Series["Series6"].Points.AddXY((k * T0), u2[k]);
                        chart1.Series["Series7"].Points.AddXY((k * T0), w1[k]);
                        chart1.Series["Series8"].Points.AddXY((k * T0), w2[k]);

                        //this.textBox1.Text = c1[0, 0].ToString();
                        if (_selectedIdentification_spec == 0)
                        {
                           
                            switch (_selectedIdentificationType)
                            {
                                case 0:

                                    Theta_h1 = matrixTranspose(updateTheta(d_h1, c1, y1, y2, u1, y1, Theta_h1, k, true, _selectedIdentificationType), this.param, 1);
                                    c1 = updateCovariance(d_h1, c1, y1, y2, u1, Theta_h1, k, _selectedIdentificationType);
                                    Theta_h2 = matrixTranspose(updateTheta(d_h2, c2, y1, y2, u2, y2, Theta_h2, k, true, _selectedIdentificationType), this.param, 1);
                                    c2 = updateCovariance(d_h2, c2, y1, y2, u2, Theta_h2, k, _selectedIdentificationType);
                                    break;
                                case 1:
                                    Theta_h1 = matrixTranspose(updateTheta(d_h1, c1, y1, y2, u1, y1, Theta_h1, k, true, _selectedIdentificationType), this.param, 1);
                                    c1 = updateCovariance(d_h1, c1, y1, y2, u1, Theta_h1, k, _selectedIdentificationType);
                                    Theta_h2 = matrixTranspose(updateTheta(d_h2, c2, y1, y2, u2, y2, Theta_h2, k, true, _selectedIdentificationType), this.param, 1);
                                    c2 = updateCovariance(d_h2, c2, y1, y2, u2, Theta_h2, k, _selectedIdentificationType);
                                    break;
                                case 2:
                                    double[,] Theta_cop1 = Theta_h1;
                                    double[,] Theta_cop2 = Theta_h2;

                                    Theta_h1 = matrixTranspose(updateTheta(d_h1, c1, y1, y2, u1, y1, Theta_h1, k, true, _selectedIdentificationType), this.param, 1);
                                    c1 = updateCovarianceAdaptive(d_h1, c1, y1, y2, u1, y1, Theta_cop1, k, true, _selectedIdentificationType, ref phi_adaptive_h1, ref lambda_adaptive_h1, ref ny_adaptive_h1);
                                    Theta_h2 = matrixTranspose(updateTheta(d_h2, c2, y1, y2, u2, y2, Theta_h2, k, true, _selectedIdentificationType), this.param, 1);
                                    c2 = updateCovarianceAdaptive(d_h2, c2, y1, y2, u2, y2, Theta_cop2, k, true, _selectedIdentificationType, ref phi_adaptive_h2, ref lambda_adaptive_h2, ref ny_adaptive_h2);

                                    break;
                                default:
                                    break;
                            }
                        }
                        else
                        {
                            try
                            {
                                u1f[k + 1] = u1f[k] + T0 * (u1[k] - filter_co * u1f[k]);
                                u2f[k + 1] = u2f[k] + T0 * (u2[k] - filter_co * u2f[k]);
                                y1f[k + 1] = y1f[k] + T0 * (y1[k] - filter_co * y1f[k]);
                                y2f[k + 1] = y2f[k] + T0 * (y2[k] - filter_co * y2f[k]);
                            }
                            catch (Exception ee)
                            {
                                timer1.Enabled = false;
                                WarnDialog1.Show("Warn message",
                                   "&" + ee.ToString().Remove(ee.ToString().IndexOf(":")));
                            }
                            switch (_selectedIdentificationType)
                            {
                                case 0:
                                    Theta_h1 = matrixTranspose(updateTheta(d_h1, c1, y1f, y2f, u1f, y1f, Theta_h1, k, true, _selectedIdentificationType), this.param, 1);
                                    c1 = updateCovariance(d_h1, c1, y1f, y2f, u1f, Theta_h1, k, _selectedIdentificationType);
                                    Theta_h2 = matrixTranspose(updateTheta(d_h2, c2, y1f, y2f, u2f, y2f, Theta_h2, k, true, _selectedIdentificationType), this.param, 1);
                                    c2 = updateCovariance(d_h2, c2, y1f, y2f, u2f, Theta_h2, k, _selectedIdentificationType);
                                    break;
                                case 1:

                                    Theta_h1 = matrixTranspose(updateTheta(d_h1, c1, y1f, y2f, u1f, y1f, Theta_h1, k, true, _selectedIdentificationType), this.param, 1);
                                    c1 = updateCovariance(d_h1, c1, y1f, y2f, u1f, Theta_h1, k, _selectedIdentificationType);
                                    Theta_h2 = matrixTranspose(updateTheta(d_h2, c2, y1f, y2f, u2f, y2f, Theta_h2, k, true, _selectedIdentificationType), this.param, 1);
                                    c2 = updateCovariance(d_h2, c2, y1f, y2f, u2f, Theta_h2, k, _selectedIdentificationType);
                                    break;
                                case 2:
                                    double[,] Theta_cop1 = Theta_h1;
                                    double[,] Theta_cop2 = Theta_h2;

                                    Theta_h1 = matrixTranspose(updateTheta(d_h1, c1, y1f, y2f, u1f, y1f, Theta_h1, k, true, _selectedIdentificationType), this.param, 1);
                                    c1 = updateCovarianceAdaptive(d_h1, c1, y1f, y2f, u1f, y1f, Theta_cop1, k, true, _selectedIdentificationType, ref phi_adaptive_h1, ref lambda_adaptive_h1, ref ny_adaptive_h1);
                                    Theta_h2 = matrixTranspose(updateTheta(d_h2, c2, y1f, y2f, u2f, y2f, Theta_h2, k, true, _selectedIdentificationType), this.param, 1);
                                    c2 = updateCovarianceAdaptive(d_h2, c2, y1f, y2f, u2f, y2f, Theta_cop2, k, true, _selectedIdentificationType, ref phi_adaptive_h2, ref lambda_adaptive_h2, ref ny_adaptive_h2);

                                    break;
                                default:
                                    break;
                            }

                        }
                        
                        // Identifikovane parametry jednotlivych Theta
                        b01[k + 1] = Theta_h1[0, 0];
                        a01[k + 1] = Theta_h1[0, 1];
                        a02[k + 1] = Theta_h1[0, 2];

                        b04[k + 1] = Theta_h2[0, 0];
                        a03[k + 1] = Theta_h2[0, 1];
                        a04[k + 1] = Theta_h2[0, 2];
                        //this.textBox1.Text = Theta_h1[0, 0].ToString();
                        //this.textBox2.Text = Theta_h1[0, 1].ToString();
                        //-------------------------------------------------------------------------
                        // Synteza regulatoru - polynomialni matice
                        //-------------------------------------------------------------------------
                        controller[0] = 1;  // p01
                        controller[1] = 0;  // p02
                        controller[2] = 0;  // p03
                        controller[3] = 1;  // p04
                        controller[4] = (this.alpha1 * this.alpha1) / (Theta_h1[0, 0]);  // t01
                        controller[5] = (1 / Theta_h1[0, 0]) * (2 * this.alpha1 - Theta_h1[0, 1]);  //t11
                        controller[6] = 0;  //t02
                        controller[7] = 0;  //t03
                        controller[8] = -(Theta_h1[0, 2] / Theta_h1[0, 0]); //t12
                        controller[9] = -(Theta_h2[0, 1] / Theta_h2[0, 0]); //t13
                        controller[10] = (this.alpha2 * this.alpha2) / Theta_h2[0, 0];  //t04
                        controller[11] = (1 / Theta_h2[0, 0]) * (2 * this.alpha2 - Theta_h2[0, 2]);  //t14
                        //-------------------------------------------------------------------------
                        z11[k + 1] = z11[k] + T0 * e1[k];
                        e1[k + 1] = w1[k + 1] - y1[k + 1];
                        z14[k + 1] = z14[k] + T0 * e2[k];
                        e2[k + 1] = w2[k + 1] - y2[k + 1];
                        UR[0] = ((controller[4] * z11[k + 1] + (gamma1 * controller[5]) * e1[k + 1])) + (gamma1 * controller[8] * e2[k + 1]);
                        UR[1] = ((controller[10] * z14[k + 1]) + (gamma2 * controller[11]) * e2[k + 1]) + (gamma2 * controller[9] * e1[k + 1]);
                        UQ[0] = ((1 - gamma1) * controller[5] * y1[k + 1]) + ((1 - gamma1) * controller[8] * y2[k + 1]);
                        UQ[1] = ((1 - gamma2) * controller[9] * y1[k + 1]) + ((1 - gamma2) * controller[11] * y2[k + 1]); ;
                        //-------------------------------------------------------------------------
                        u1[k + 1] = UR[0] - UQ[0];
                        u2[k + 1] = UR[1] - UQ[1];
                        //this.textBox1.Text = u1[k].ToString();
                        //this.textBox2.Text = u2[k].ToString();
                        q1f[k + 1] = u1[k + 1] + q1fs;
                        q2f[k + 1] = u2[k + 1] + q2fs;

                        this.toolStripProgressBar1.PerformStep();
                        this.k = this.k + 1;
                    }
                    else
                    {
                        whichExport = exportCSV((int)(_simulationtime / T0), false);
                        toolStripProgressBar1.Value = toolStripProgressBar1.Minimum;
                        radioButton4.Enabled = true;
                        radioButton5.Enabled = true;

                        timer1.Enabled = false;

                    }





                    break;
                default:
                    break;
            }
          
        }

        private void exponencialForgettingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string value = _phi_coef.ToString();
            double member;
            if (InputBox.Show("Exponencial forgetting parameters",
                "&Insert parameter φ:", ref value) == DialogResult.OK)
            {
                if (double.TryParse(value.ToString(), out member) == true && timer1.Enabled == false)
                {
                    this._phi_coef = double.Parse(value.ToString());
                }
            }
        }

        private void adaptiveDirectionalForgettingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bool chck = true;
            string[] value = { adaptiveDirectionalBackupParameters[0].ToString(), adaptiveDirectionalBackupParameters[1].ToString(), adaptiveDirectionalBackupParameters[2].ToString(), adaptiveDirectionalBackupParameters[3].ToString() };
            double member;

            if (InputBox2.Show("Adaptive directional forgetting parameters", new string[] { "&Insert φ:", "&Insert λ:", "&Insert ρ:", "Insert ν:" }, ref value) == DialogResult.OK)
            {
                foreach (string s in value)
                {
                    if(double.TryParse(s, out member)== false)
                    {
                        chck = false;
                    }
                }
                if (chck == true && timer1.Enabled == false)
                {
                    adaptiveDirectionalBackupParameters[0] = double.Parse(value[0].ToString());
                    adaptiveDirectionalBackupParameters[1] = double.Parse(value[1].ToString());
                    adaptiveDirectionalBackupParameters[2] = double.Parse(value[2].ToString());
                    adaptiveDirectionalBackupParameters[3] = double.Parse(value[3].ToString());

                }
                else
                {
                    // vrat chybove hlaseni
                }
            }
        }

        private void pictureBox2_MouseDown(object sender, MouseEventArgs e)
        {
            pictureBox2.BackgroundImage = Image.FromFile("..\\..\\img\\close_pressed.png");
        }

        private void pictureBox2_MouseUp(object sender, MouseEventArgs e)
        {
            pictureBox2.BackgroundImage = Image.FromFile("..\\..\\img\\close_default.png");
        }

        private void pictureBox1_MouseUp(object sender, MouseEventArgs e)
        {
            pictureBox1.BackgroundImage = Image.FromFile("..\\..\\img\\run_default.png");
        }

        private void pictureBox1_MouseDown(object sender, MouseEventArgs e)
        {
            pictureBox1.BackgroundImage = Image.FromFile("..\\..\\img\\run_pressed.png");
        }

        private void radioButton4_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void label2_Click(object sender, EventArgs e)
        {

        }

        private void label1_Click(object sender, EventArgs e)
        {

        }

        private void label4_Click(object sender, EventArgs e)
        {

        }

        private void label5_Click(object sender, EventArgs e)
        {

        }

        private void label7_Click(object sender, EventArgs e)
        {

        }

        private void pictureBox2_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        private void clearGraph()
        {
            chart1.Series["Series1"].Points.Clear();
            chart1.Series["Series2"].Points.Clear();
            chart1.Series["Series3"].Points.Clear();
            chart1.Series["Series4"].Points.Clear();
            chart1.Series["Series5"].Points.Clear();
            chart1.Series["Series6"].Points.Clear();
            chart1.Series["Series7"].Points.Clear();
            chart1.Series["Series8"].Points.Clear();
        }
        private void pictureBox1_Click(object sender, EventArgs e)
        {
            this.k = 0;
            if (radioButton5.Checked == true)
            {
                initialControlling();
                initialIdentification();
                initialAdaptiveControlling();
                _selectedIdentification_spec = comboBox1.SelectedIndex;
                //desiredLevels();
                if (checkDesiredCharacter(convert_arr(textBox5.Text.ToString()), convert_arr(textBox6.Text.ToString()), this.T0, this.maxLevelh1) == 0 && checkDesiredCharacter(convert_arr(textBox7.Text.ToString()), convert_arr(textBox8.Text.ToString()), this.T0, this.maxLevelh2) == 0)
                {
                    // set desired vector tank1
                    desiredTime1 = convert_arr(textBox5.Text.ToString());
                    waterLevel1 = convert_arr(textBox6.Text.ToString());
                    // set desired vector tank2
                    desiredTime2 = convert_arr(textBox7.Text.ToString());
                    waterLevel2 = convert_arr(textBox8.Text.ToString());
                    double _simulationTime_1 = desiredTime1[desiredTime1.Length - 1];
                    double _simulationTime_2 = desiredTime2[desiredTime2.Length - 1];
                    //MessageBox.Show(_simulationTime.ToString());
                    
                    
                    chart1.ChartAreas["ChartArea1"].AxisX.Maximum = Math.Max(desiredTime1[desiredTime1.Length - 1], desiredTime2[desiredTime2.Length - 1]);

                    w1 = new double[(int)(Math.Max(desiredTime1[desiredTime1.Length - 1], desiredTime2[desiredTime2.Length - 1]) / T0) + 2];
                    w2 = new double[(int)(Math.Max(desiredTime1[desiredTime1.Length - 1], desiredTime2[desiredTime2.Length - 1]) / T0) + 2];

                    generatigW_value(desiredTime1, waterLevel1, w1, _simulationTime_1);
                    generatigW_value(desiredTime2, waterLevel2, w2, _simulationTime_2);


                    _reAllocationVariablesControl((int)(Math.Max(desiredTime1[desiredTime1.Length - 1], desiredTime2[desiredTime2.Length - 1]) / T0));

                   _simulationtime = Math.Max(desiredTime1[desiredTime1.Length - 1], desiredTime2[desiredTime2.Length - 1]);
                       
                    // change maximum of toolStripProgressBar
                    this.toolStripProgressBar1.Maximum = (int)(_simulationtime / T0);

                    //----------------------------------------------------------------------
                    //InitTheta(Theta_h1, new double[] { 1.0083, 0.2373, -0.2356 }, 1, param);
                    matrixEye(c1, 1000, this.param, this.param);
                    //InitTheta(Theta_h2, new double[] { 1.6864, -0.3856, 0.5869 }, 1, param);
                    matrixEye(c2, 1000, this.param, this.param);
                    clearGraph();
                    timer1.Enabled = true;
                }
                else
                {
                    generateErrorMessage(checkDesiredCharacter(convert_arr(textBox5.Text.ToString()), convert_arr(textBox6.Text.ToString()), this.T0, this.maxLevelh1), checkDesiredCharacter(convert_arr(textBox7.Text.ToString()), convert_arr(textBox8.Text.ToString()), this.T0, this.maxLevelh2));

                }
            }
            else
            {
                clearGraph();
                initialIdentification();
                _selectedIdentification_spec = comboBox1.SelectedIndex;
                _reAllocationVariablesIdent((int)(_simulationtime / T0));
                //MessageBox.Show("Ahoj adame");
                //this.textBox3.Text = ((int)(_simulationtime / T0)).ToString();
                timer1.Enabled = true;
            }
            
        }
        // start testing Input into repating sequence
        public bool checkTime(double[] parTime, double period)
        {
            bool returnValue = true;
            for (int i = 0; i < parTime.Length; i++)
            {
                double pomValue = parTime[i] / period;
                if ((Convert.ToInt32(pomValue) * period) != (parTime[i]))
                {
                    returnValue = false;
                    break;
                }

            }
            return returnValue;
        }
        public bool checkTimeSequence(double[] parTime)
        {
            bool returnValue = true;
            for (int i = 0; i < parTime.Length - 1; i++)
            {
                if ((parTime[i] > parTime[i + 1]))
                {
                    returnValue = false;
                    break;
                }
            }
            return returnValue;
        }
        public bool sameValueTime(double[] parTime, double[] parValues)
        {
            bool returnValue = true;
            for (int i = 0; i < parTime.Length - 1; i++)
            {
                if (parTime[i] == parTime[i + 1])
                {
                    if (parValues[i] == parValues[i + 1])
                    {
                        returnValue = false;
                        break;
                    }
                }

            }
            return returnValue;
        }
        public bool countTimeValues(double[] parTime)
        {
            bool returnValue = true;
            int count;
            for (int i = 0; i < parTime.Length - 1; i++)
            {
                count = 1;
                for (int j = i + 1; j < parTime.Length; j++)
                {
                    if (parTime[i] == parTime[j])
                    {
                        count++;
                    }
                    if (count > 2)
                    {
                        returnValue = false;
                        break;
                    }
                }
            }
            return returnValue;
        }
        public bool BoundedLimitValues(double[] parValues, double upperLimit)
        {
            bool returnValue = true;
            for (int i = 0; i < parValues.Length; i++)
            {
                if (parValues[i] > upperLimit || parValues[i] < 0)
                {
                    returnValue = false;
                    break;
                }
            }
            return returnValue;
        }
        public bool tryParseInput(string parValue)
        {
            bool returnValue = true;
            string[] stringValues = parValue.Split('-');
            double number;
            for (int i = 0; i < stringValues.Length; i++)
            {
                if (Double.TryParse(stringValues[i], out number) == false)
                {
                    returnValue = false;
                    break;
                }

            }

            return returnValue;
        }
        public double[] convert_arr(string parValue)
        {
            double[] arr;
            if (tryParseInput(parValue) == true)
            {
                arr = Array.ConvertAll(parValue.Split('-'), Double.Parse);
            }
            else
            {
                arr = null;
            }
            return arr;
        }

         
        public int checkDesiredCharacter(double[] parTime, double[] parValues, double period,double upperLimit)
        {
            int errorType = 0;

            // Try Parse -  pokud nejde prevest -> Vrat chybu
            if (parTime == null || parValues == null)
            {
                errorType = 1;
            }
            else
            {
                // 0. Stejná délka obou polí
                if (parTime.Length != parValues.Length)
                {
                    errorType = 2;
                }
                // 1. Je-li hodnota v časovém vektoru mimo násobek periody vrat chybu o kroku
                if (checkTime(parTime, period) != true)  // Do formu zmenit na periodu vzorkovani
                {
                    errorType = 3;
                }
                // 2. Posloupnost časových hodnot
                if (checkTimeSequence(parTime) != true)
                {
                    errorType = 4;
                }
                // 3. První člen nulový
                if (parTime[0] != 0)
                {
                    errorType = 5;
                }
                // Omezené hodnoty hladin
                if (BoundedLimitValues(parValues, upperLimit) != true)
                {
                    errorType = 6;
                }
                // 4. Ve stejnych časových okamžicích nesmí být stejné hodnoty hladiny
                if (sameValueTime(parTime, parValues) != true)
                {
                    errorType = 7;
                }
                // 5. vicenásobné používání časových okamžiků
                if (countTimeValues(parTime) != true)
                {
                    errorType = 8;
                }
                
            }
            return errorType;
        }
        public void generateErrorMessage(int mess1, int mess2)
        {
            // switch error
            Hashtable messagesWarn = new Hashtable();
            messagesWarn.Add(0, "Correct");
            messagesWarn.Add(1, "Invalid characters");
            messagesWarn.Add(2, "Must be the same length");
            messagesWarn.Add(3, "Time value is out of period time.");
            messagesWarn.Add(4, "Must be monotonically increasing");
            messagesWarn.Add(5, "The first value of time vector \n must be equal to zero");
            messagesWarn.Add(6, "Output vector out of range");
            messagesWarn.Add(7, "Duplication in output vector");
            //messagesWarn.Add(8, "Invalid characters");
            messagesWarn.Add(8, "Multiple use of time");
            //messagesWarn.Add(9, "Different length of simulation time");
            string _const1 = "Tank 1";
            string _const2 = "Tank 2";
            string generate_mess1 = "";
            string generate_mess2 = "";
            generate_mess1 = _const1 + ": "+ messagesWarn[mess1].ToString();
            generate_mess2 = _const2 + ": "+ messagesWarn[mess2].ToString();
            WarnDialog2.Show("Warn message",
                "&"+generate_mess1,"&"+generate_mess2);
        }
        //------------------------------------------------------------------------------------------------------------------
        // end testing Input into repeating sequence
        //------------------------------------------------------------------------------------------------------------------
        // generating w1,w2 vector
        //------------------------------------------------------------------------------------------------------------------
        private void generatigW_value(double[] _parTime, double[] _parValue,double[] saveWvalue,double _simulationTime)
        {
            double smernice = 0;
            int index1 = 0;
            for (int krok = 0; krok <= (int)(_simulationTime / T0); krok++)
            {
                //{0,40,40,80}
                //{0.2,0.2,0.1,0.1}
                if (krok != (int)(_simulationTime / T0))
                {
                    if ((krok) * T0 == _parTime[index1])
                    {

                        if ((krok) * T0 == _parTime[index1 + 1])
                        {
                            // step
                            index1++;
                            saveWvalue[krok] = _parValue[index1];
                            smernice = 0;
                        }
                        else
                        {
                            // line
                            if (waterLevel1[index1] == _parValue[index1 + 1])
                            {
                                saveWvalue[krok] = _parValue[index1];
                                smernice = 0;
                            }
                            else
                            {
                                // ramp
                                saveWvalue[krok] = _parValue[index1];
                                smernice = (_parValue[index1 + 1] - _parValue[index1]) / (_parTime[index1 + 1] - _parTime[index1]);
                            }
                        }
                        index1++;
                    }
                    else
                    {
                        saveWvalue[krok] = _parValue[index1 - 1] + smernice * T0 * (krok - _parTime[index1 - 1] / T0);
                    }

                }
                else
                {
                    saveWvalue[krok] = _parValue[index1];
                }
                //chart1.Series["Series1"].Points.AddXY(krok * T0, w1[krok]);
                //krok++;
            }
        }
        //----------------------------------------------------------------------------------
        // end generating w1,w2 vector


        //----------------------------------------------------------------------------------
        // realocation according simulation time
        //----------------------------------------------------------------------------------
        private void _reAllocationVariablesControl(int _parStep)
        {
            //------------------------------------------------------------------------------
            // Tank 1
            //------------------------------------------------------------------------------
            h1 = new double[_parStep + 2];
            h1[0] = h1s;
            u1 = new double[_parStep + 2];
            u1[0] = 0;
            q1f = new double[_parStep + 2];
            q1f[0] = q1fs + u1[0];
            y1 = new double[_parStep + 2];
            y1[0] = 0;           
            e1 = new double[_parStep + 2];
            e1[0] = w1[0] - y1[0];
            y1f = new double[_parStep + 2];
            y1f[0] = 0;
            //------------------------------------------------------------------------------
            // Tank 2
            //------------------------------------------------------------------------------
            h2 = new double[_parStep + 2];
            h2[0] = h2s;
            u2 = new double[_parStep + 2];
            q2f = new double[_parStep + 2];
            q2f[0] = q2fs + u2[0];
            y2 = new double[_parStep + 2];
            y2f = new double[_parStep + 2];
            y2f[0] = 0;
            e2 = new double[_parStep + 2];
            e2[0] = w2[0] - y2[0];
            q1 = new double[_parStep + 2];
            q2 = new double[_parStep + 2];
            // special controlling variables
            z11 = new double[_parStep + 2];
            z14 = new double[_parStep + 2];
            q1[0] = k1 * Math.Sign(h1[0] - h2[0]) * Math.Sqrt(Math.Abs(h1[0] - h2[0]));
            q2[0] = k2 * Math.Sqrt(h2[0]);

            Theta_h1 = new double[1, param];
            InitTheta(Theta_h1, new double[] { Theta_backup[0], Theta_backup[1], Theta_backup[2] }, 1, param);
            Theta_h2 = new double[1, param];
            InitTheta(Theta_h2, new double[] { Theta_backup[3], Theta_backup[4], Theta_backup[5] }, 1, param);

            matrixEye(c1, 1000, this.param, this.param);
            matrixEye(c2, 1000, this.param, this.param);

            b01 = new double[_parStep + 2];
            b01[0] = Theta_h1[0, 0];
            a01 = new double[_parStep + 2];
            a01[0] = Theta_h1[0, 1];
            a02 = new double[_parStep + 2];
            a02[0] = Theta_h1[0, 2];
            b04 = new double[_parStep + 2];
            b04[0] = Theta_h2[0, 0];
            a03 = new double[_parStep + 2];
            a03[0] = Theta_h2[0, 1];
            a04 = new double[_parStep + 2];
            a04[0] = Theta_h2[0, 2];


        }
        private void _reAllocationVariablesIdent(int _parStep)
        {
            h1 = new double[_parStep + 2];
            h1[0] = h1s;
            u1 = new double[_parStep + 2];
            u1[0] = _actuator1;
            q1f = new double[_parStep + 2];
            q1f[0] = q1fs + u1[0];
            y1 = new double[_parStep + 2];
            y1[0] = 0;

            y1f = new double[_parStep + 2];
            y1f[0] = 0;

            h2 = new double[_parStep + 2];
            h2[0] = h2s;
            u2 = new double[_parStep + 2];
            u2[0] = _acturator2;
            q2f = new double[_parStep + 2];
            q2f[0] = q2fs + u2[0];
            y2 = new double[_parStep + 2];
            y2[0] = 0;
            y2f = new double[_parStep + 2];
            y2f[0] = 0;

            q1 = new double[_parStep + 2];
            q2 = new double[_parStep + 2];

            q1[0] = k1 * Math.Sign(h1[0] - h2[0]) * Math.Sqrt(Math.Abs(h1[0] - h2[0]));
            q2[0] = k2 * Math.Sqrt(h2[0]);


            Theta_h1 = new double[1, param];
            InitTheta(Theta_h1, new double[] { Theta_backup[0], Theta_backup[1], Theta_backup[2] }, 1, param);
            Theta_h2 = new double[1, param];
            InitTheta(Theta_h2, new double[] { Theta_backup[3], Theta_backup[4], Theta_backup[5] }, 1, param);

            matrixEye(c1, 1000, this.param, this.param);
            matrixEye(c2, 1000, this.param, this.param); 

            b01 = new double[_parStep + 2];
            b01[0] = Theta_h1[0, 0];
            a01 = new double[_parStep + 2];
            a01[0] = Theta_h1[0, 1];
            a02 = new double[_parStep + 2];
            a02[0] = Theta_h1[0, 2];
            b04 = new double[_parStep + 2];
            b04[0] = Theta_h2[0, 0];
            a03 = new double[_parStep + 2];
            a03[0] = Theta_h2[0, 1];
            a04 = new double[_parStep + 2];
            a04[0] = Theta_h2[0, 2];



        }

        private void constantOfFilterToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string value = this.filter_co.ToString();
            double member;
            if (InputBox.Show("Filter constant c0",
                "&Insert  filter parameter c0:", ref value) == DialogResult.OK)
            {
                if (double.TryParse(value.ToString(), out member) == true && timer1.Enabled == false)
                {
                    this.filter_co = double.Parse((Math.Round(double.Parse(value), 2).ToString()));
                }
            }
        }

        private void initializeTheta1ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bool chck = true;
            string[] value = { Theta_backup[0].ToString(), Theta_backup[1].ToString(), Theta_backup[2].ToString() };
            double member;
            if (InputBox3.Show("Initialize vector Theta 1",
                new string[] { "&Insert b01:", "&Insert a01:", "&Insert a02:"}, ref value) == DialogResult.OK)
            {
                foreach (string s in value)
                {
                    if (double.TryParse(s, out member) == false)
                    {
                        chck = false;
                    }
                }
                if (chck == true && timer1.Enabled == false)
                {
                    Theta_backup[0] = double.Parse(value[0].ToString());
                    Theta_backup[1] = double.Parse(value[1].ToString());
                    Theta_backup[2] = double.Parse(value[2].ToString());

                }
                else
                {
                    //Chyba
                }
            }
        }

        private void initializeTheta2ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bool chck = true;
            string[] value = { Theta_backup[3].ToString(), Theta_backup[4].ToString(), Theta_backup[5].ToString() };
            double member;
            if (InputBox3.Show("Initialize vector Theta 1",
                new string[] { "&Insert b04:", "&Insert a03:", "&Insert a04:" }, ref value) == DialogResult.OK)
            {
                foreach (string s in value)
                {
                    if (double.TryParse(s, out member) == false)
                    {
                        chck = false;
                    }
                }
                if (chck == true && timer1.Enabled == false)
                {
                    Theta_backup[3] = double.Parse(value[0].ToString());
                    Theta_backup[4] = double.Parse(value[1].ToString());
                    Theta_backup[5] = double.Parse(value[2].ToString());
                }
                else
                {
                    // chyba
                }
            }
        }
        //----------------------------------------------------------------------------------
        // end realocation
        //----------------------------------------------------------------------------------
       




    }
}
